%%%% Created by Amin Rahmani, Ph.D., P.E. - May, 2022
%%%% Copyright © 2022 Amin Rahmani. All rights reserved.

function [Time, block_accel, block_disp, sliding_vel, max_disp, T, degraded_G_ratio, strain]=Flexible_Sliding_Block_Model_Sliding_2_Way(accelfile, ky, rho, Vs, H, Mw, shearstrain, GG_max, Damping, dir)
%   [Time, block_accel, block_disp, sliding_vel, max_disp, T, degraded_G_ratio, strain]=Flexible_Sliding_Block_Model_1(accelfile, ky, rho, Vs, H, Mw, shearstrain, GG_max, Damping, dir)
%   Modified Newmark Sliding Block Analysis (Kramer and Smith, 1997) -
%   Sliding in both + and - directions
%
%
%   Model 2 - THis model considers sliding in both + and - directions. This
%   model should not be used for computing downslope ground movement since it considers
%   sliding upslope too.
%   
%
%   Input :   accelfile = Accelration time history in 'g' with format of 8
%                         columns per row
%             ky  = yield acceleration array in g, positive
%             rho = mass density of sliding block in lb.mass/ft^3
%             Vs  = Average shear wave velocity in ft/s
%             H   = Effective Height of the equivalent sliding mass, ft,
%                   See Bray and Macedo (2021)
%             Mw = Earthquake magnitude
%             shearstrain = Shear Strain in decimal
%             GG_max = dynamic G/G_max reduction curve
%             Damping = Array of Damping in decimal
%             dir = direction of ground motion --- 1 for positive direction
%                                                  -1 for negative
%                                                  direction
%             
%   Output:   Time = Time Hisotry
%             block_accel = Absolute acceleration of the sliding mass
%                           in g
%             block_disp =  Displacment of the sliding mass
%                           in inch with respect to the base
%             sliding_vel = sliding mass velocity in in/s
%             max_disp =    max displacement in inch 
%             T = Period at each iteration for each ky value 
%             degraded_G_ratio = degraded G ratio
%             strain = computed  shear strain


%%%% Newmark-beta Parameters for the time integration scheme
beta=0.25;
gamma=0.5;

%%%%% Define gravity
gravity=32.2; %%%% ft/sec^2

%%%% Upper Mass (M) and Lower Mass (Mo)
MT=rho*H*1*1; %%% Total mass of block
M=rho*1*1*(H/2); %%%%% Generalized mass for a column of soil assuming 1st mode vibration Phi_n=sin((2*n-1)*pi*z/(2H)), z from bottom of soil column base
Mo=MT-M; %%%% Mass of base block


%%%% Declariation of 1st element of arrrays
d_ug(1)=0;
ug(1)=0;

u(1)=0;
d_u(1)=0;
dd_u(1)=0;

%%%%%%%%%%%% Define Seed Motions %%%%%%%%%%%
%%%%%% The following lines down to Line 83 may need to be revised based on
%%%%%% the format of input ground motion files.

fid = fopen(accelfile,'r') ;  % Read acceleration time history
c1=readtextline(fid);   % Read and skip line 1
n=readmatrix(fid,1,1);  % Read number of data points in the 2nd line
dt=readmatrix(fid,1,1); % Read time step in the 2nd line

for i=1:n
    %%% Acceleration 
    Accel(i)=fscanf(fid,'%g ',1);

    %%% Time
    Time(i)=(i-1)*dt;

end

fclose('all');

%%% Base Motion in ft/sec^2
dd_ug = (Accel).*(dir*gravity); %% Use negative, i.e., dir=-1, to be consistent with the shaking direction used in Newmark Rigid Block Sliding

%%% Determine Time Step
dt=Time(1,2)-Time(1,1);

%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Modified Newmark (Kramer and Smith 1997) - Equivalent Linear Elastic Model
%%%%%%%%%%%%%%%%%%%%%%

for m=1:length(ky)
        
    %%% Initiating Parameters
    difference_G=1e1;
    difference_D=1e1;
    tol=1e-5;
    
    G=ones(length(ky),200);
    T=ones(length(ky),200);
    zeta=ones(length(ky),200);
    
    for p=1:length(ky)
        G(p,1)=(Vs^2)*rho;
    end
    
    zeta(m,1)=Damping(1);
    
    incr=1;    
    
    while (difference_G > tol || difference_D > tol) && incr < 200
        
        dd_s{m}=zeros(1,length(Accel));
        d_s{m}=zeros(1,length(Accel));
        s{m}=zeros(1,length(Accel));
        
        %%%% SDOF Stiffness
        Vs_eq=(G(m,incr)/rho)^0.5;
        T(m,incr)=4*H/Vs_eq;
        K(m,incr)=M/((4*H/(Vs_eq*2*3.1416))^2);
        K=K(m,incr);
        
        %%%% SDOF Damping Coeff.
        C=2*zeta(m,incr)*(K*M)^(1/2);
       
        for i=1:(length(Accel)-1)
      
            d_ug(i+1)= d_ug(i)+(1-gamma)*dt*dd_ug(i)+gamma*dt*dd_ug(i+1);
            ug(i+1)= ug(i)+dt*d_ug(i)+(dt^2)/2*((1-2*beta)*dd_ug(i)+2*beta*dd_ug(i+1));
                     
            %%%Shaking before sliding
            if max(abs(dd_ug))<(ky(m)*gravity)
                
                %%%%% Solving m*ddu+k*u+c*du=-m*ddug
                term1=M+C*gamma*dt+2*K*((dt^2)/2)*beta;
                term2=-M*dd_ug(i+1)-C*(d_u(i)+(1-gamma)*dt*dd_u(i))-K*(u(i)+dt*d_u(i)+(dt^2)/2*(1-2*beta)*dd_u(i));
                
                dd_u(i+1)=term2/term1;
                d_u(i+1)= d_u(i)+(1-gamma)*dt*dd_u(i)+gamma*dt*dd_u(i+1);
                u(i+1)= u(i)+dt*d_u(i)+(dt^2)/2*((1-2*beta)*dd_u(i)+2*beta*dd_u(i+1));
                
            else
                %%%%% Assume there is no sliding initially
                
                %%%%% Sloving m*ddu+k*u+c*du= -m*ddug
                term1=M+C*gamma*dt+2*K*((dt^2)/2)*beta;
                term2=-M*dd_ug(i+1)-C*(d_u(i)+(1-gamma)*dt*dd_u(i))-K*(u(i)+dt*d_u(i)+(dt^2)/2*(1-2*beta)*dd_u(i));
                
                dd_u(i+1)=term2/term1;
                d_u(i+1)= d_u(i)+(1-gamma)*dt*dd_u(i)+gamma*dt*dd_u(i+1);
                u(i+1)= u(i)+dt*d_u(i)+(dt^2)/2*((1-2*beta)*dd_u(i)+2*beta*dd_u(i+1));
                
                dd_s{m}(i+1)=0;
                d_s{m}(i+1)= 0;
                s{m}(i+1)= s{m}(i);

                if abs(-(M+Mo)*dd_ug(i+1)-M*dd_u(i+1)) >= ky(m)*(M+Mo)*gravity
                
                    %%% Shaking during sliding

                    %%%%% Sloving equations (3), (4) and (5) of Kramer and
                    %%%%% Smith (1997)
                    term3=M+C*gamma*dt+K*(dt^2)/2*2*beta-(M^2)/(M+Mo);
                    term4=M*ky(m)*gravity-C*(d_u(i)+(1-gamma)*dt*dd_u(i))-K*(u(i)+dt*d_u(i)+((dt^2)/2)*(1-2*beta)*dd_u(i));

                    dd_u(i+1)=term4/term3;
                    d_u(i+1)= d_u(i)+(1-gamma)*dt*dd_u(i)+gamma*dt*dd_u(i+1);
                    u(i+1)= u(i)+dt*d_u(i)+(dt^2)/2*((1-2*beta)*dd_u(i)+2*beta*dd_u(i+1));

                    %%%%% Sliding Mass Accel, Vel, and Disp
                    dd_s{m}(i+1)=-dd_ug(i+1)-ky(m)*gravity-(M/(M+Mo))*dd_u(i+1);
                    d_s{m}(i+1)= d_s{m}(i)+(1-gamma)*dt*dd_s{m}(i)+gamma*dt*dd_s{m}(i+1);
                    s{m}(i+1)= s{m}(i)+dt*d_s{m}(i)+(dt^2)/2*((1-2*beta)*dd_s{m}(i)+2*beta*dd_s{m}(i+1));       
 
                end
                                
            end

        end
        
        incr=incr+1;
        
        %%%%% Equivalent shear strain
        strain(m,incr)=min(0.1,((Mw-1)/10)*max(abs(u))/H); %%% Use one-half of the soil profile height since SDOF represents center of gravity
        
        for n=1:(length(GG_max)-1)
            if strain(m,incr) >= shearstrain(n) && strain(m,incr)<shearstrain(n+1)
     
                G(m,incr)=(((strain(m,incr)-shearstrain(1,n))/(shearstrain(1,n+1)-shearstrain(1,n)))*(GG_max(1,n+1)-GG_max(1,n))+GG_max(1,n))*G(1,1);
                zeta(m,incr)=((strain(m,incr)-shearstrain(1,n))/(shearstrain(1,n+1)-shearstrain(1,n)))*(Damping(1,n+1)-Damping(1,n))+Damping(1,n);
            end        
        end

        difference_G=abs(G(m,incr)-G(m,incr-1));
        difference_D=abs(zeta(m,incr)-zeta(m,incr-1));
        
        degraded_G_ratio(m,incr)=G(m,incr)/G(1,1);

    end
    
    %%% Calculate sliding block dispalcement TH
    for i=1:length(u)
        if u(i)>0
            block_disp(i,m) = (s{m}(i)+ u(i))*12;
        else
            block_disp(i,m) = (s{m}(i))*12;
        end
    end
    
    %%% Calculate sliding block velocity TH
    sliding_vel=(d_s{m}(1,:).*12).';
    
    max_disp(1,m)=max(block_disp(:,m));
    block_accel(:,m)=(dd_u.'+dd_s{1,m}(1,:).'+dd_ug.')./gravity;
    
    
end
